@@ -9,7 +9,7 @@ import numpy as np
 
 from napoleontoolbox.neural_net import roll_multi_layer_lstm
 from napoleontoolbox.file_saver import dropbox_file_saver
-from napoleontoolbox.neural_net import roll_multi_layer_perceptron
+from napoleontoolbox.neural_net import roll_multi_layer_shortcut_perceptron
 from napoleontoolbox.boosted_trees import roll_lightgbm
 from napoleontoolbox.features import features_type
 from napoleontoolbox.utility import weights
@@ -124,6 +124,8 @@ class SimpleRunner(AbstractRunner):
         result = result[:len(df)]
 
         X = features[self.n_start:]
+        X_shortcut = shortcut_features[self.n_start:]
+
         y = result[self.n_start:, :, supervisors[sup]]
         df = df.iloc[self.n_start:]
 
@@ -188,44 +190,85 @@ class SimpleRunner(AbstractRunner):
 
         activation_function = utility.convertActivationType(activation_type)
 
-        if convolution == 1:
-            # the number of figures
-            input_size = X.shape[2]
-            hidden_size = int(layers[-1])
-            num_layers = int(layers[-2])
-            num_classes = y.shape[1]
-            tm = roll_multi_layer_lstm.RollMultiLayerLSTM(
-                X=X,
-                y=y,
-                num_classes=num_classes,
-                input_size=input_size,
-                hidden_size=hidden_size,
-                num_layers=num_layers,
-                # nn.Softmax/nn.Softmin can be good activations for this problem
-                x_type=neural_net_precision,
-                y_type=neural_net_precision
-                # activation_kwargs={'dim':1} # Parameter needed for nn.Softmax/nn.Softmin
-            )
-            tm.set_optimizer(nn.MSELoss, torch.optim.Adam, lr_type, lr=lr_start, betas=(0.9, 0.999), amsgrad=True)
-            tm = tm.set_roll_period(n, s, repass_steps=epochs)
-        elif convolution == 0:
-            tm = roll_multi_layer_perceptron.RollMultiLayerPerceptron(
-                X=X,
-                y=y,
-                layers=layers,
-                activation=activation_function,  # nn.Softmax/nn.Softmin can be good activations for this problem
-                x_type=neural_net_precision,
-                y_type=neural_net_precision,
-                # activation_kwargs={'dim':1} # Parameter needed for nn.Softmax/nn.Softmin
-            )
-            tm.set_optimizer(nn.MSELoss, torch.optim.Adam, lr_type, lr=lr_start, betas=(0.9, 0.999), amsgrad=True)
-            tm = tm.set_roll_period(n, s, repass_steps=epochs)
-        elif convolution == 2:
-            tm = roll_lightgbm.RollLightGbm(
-                X=X,
-                y=y
-            )
-            tm = tm.set_roll_period(n, s)
+        if self.shortcut_features:
+            if convolution == 1:
+                # the number of figures
+                input_size = X.shape[2]
+                hidden_size = int(layers[-1])
+                num_layers = int(layers[-2])
+                num_classes = y.shape[1]
+                tm = roll_multi_layer_lstm.RollMultiLayerLSTM(
+                    X=X,
+                    y=y,
+                    num_classes=num_classes,
+                    input_size=input_size,
+                    hidden_size=hidden_size,
+                    num_layers=num_layers,
+                    # nn.Softmax/nn.Softmin can be good activations for this problem
+                    x_type=neural_net_precision,
+                    y_type=neural_net_precision
+                    # activation_kwargs={'dim':1} # Parameter needed for nn.Softmax/nn.Softmin
+                )
+                tm.set_optimizer(nn.MSELoss, torch.optim.Adam, lr_type, lr=lr_start, betas=(0.9, 0.999), amsgrad=True)
+                tm = tm.set_roll_period(n, s, repass_steps=epochs)
+            elif convolution == 0:
+                tm = roll_multi_layer_shortcut_perceptron.RollMultiLayerShortCutPerceptron(
+                    X=X,
+                    X_shortcut=X_shortcut,
+                    y=y,
+                    layers=layers,
+                    activation=activation_function,  # nn.Softmax/nn.Softmin can be good activations for this problem
+                    x_type=neural_net_precision,
+                    y_type=neural_net_precision,
+                    # activation_kwargs={'dim':1} # Parameter needed for nn.Softmax/nn.Softmin
+                )
+                tm.set_optimizer(nn.MSELoss, torch.optim.Adam, lr_type, lr=lr_start, betas=(0.9, 0.999), amsgrad=True)
+                tm = tm.set_roll_period(n, s, repass_steps=epochs)
+            elif convolution == 2:
+                tm = roll_lightgbm.RollLightGbm(
+                    X=X,
+                    y=y
+                )
+                tm = tm.set_roll_period(n, s)
+        else :
+            if convolution == 1:
+                # the number of figures
+                input_size = X.shape[2]
+                hidden_size = int(layers[-1])
+                num_layers = int(layers[-2])
+                num_classes = y.shape[1]
+                tm = roll_multi_layer_lstm.RollMultiLayerLSTM(
+                    X=X,
+                    y=y,
+                    num_classes=num_classes,
+                    input_size=input_size,
+                    hidden_size=hidden_size,
+                    num_layers=num_layers,
+                    # nn.Softmax/nn.Softmin can be good activations for this problem
+                    x_type=neural_net_precision,
+                    y_type=neural_net_precision
+                    # activation_kwargs={'dim':1} # Parameter needed for nn.Softmax/nn.Softmin
+                )
+                tm.set_optimizer(nn.MSELoss, torch.optim.Adam, lr_type, lr=lr_start, betas=(0.9, 0.999), amsgrad=True)
+                tm = tm.set_roll_period(n, s, repass_steps=epochs)
+            elif convolution == 0:
+                tm = roll_multi_layer_shortcut_perceptron.RollMultiLayerPerceptron(
+                    X=X,
+                    y=y,
+                    layers=layers,
+                    activation=activation_function,  # nn.Softmax/nn.Softmin can be good activations for this problem
+                    x_type=neural_net_precision,
+                    y_type=neural_net_precision,
+                    # activation_kwargs={'dim':1} # Parameter needed for nn.Softmax/nn.Softmin
+                )
+                tm.set_optimizer(nn.MSELoss, torch.optim.Adam, lr_type, lr=lr_start, betas=(0.9, 0.999), amsgrad=True)
+                tm = tm.set_roll_period(n, s, repass_steps=epochs)
+            elif convolution == 2:
+                tm = roll_lightgbm.RollLightGbm(
+                    X=X,
+                    y=y
+                )
+                tm = tm.set_roll_period(n, s)
 
         eval_slice, test_slice, predictors, tt, tt_true = tm.unroll()
 
